@@ -1,10 +1,10 @@
-# $Id: Application.pm,v 1.30 2003/02/01 07:46:11 jesse Exp $
+# $Id: Application.pm,v 1.34 2003/06/02 12:43:18 jesse Exp $
 
 package CGI::Application;
 
 use strict;
 
-$CGI::Application::VERSION = '3.0';
+$CGI::Application::VERSION = '3.1';
 
 
 use CGI::Carp;
@@ -142,18 +142,17 @@ sub run {
         my $body = eval { $autoload_mode ? $self->$rmeth($rm) : $self->$rmeth() };
         die "Error executing run mode '$rm': $@" if $@;
 
+        # Support scalar-ref for body return
+        my $bodyref = (ref($body) eq 'SCALAR') ? $body : \$body;
+
+        # Call cgiapp_postrun() hook
+        $self->cgiapp_postrun($bodyref);
+
 	# Set up HTTP headers
 	my $headers = $self->_send_headers();
 
 	# Build up total output
-	my $output = $headers;
-
-	# Support return as SCALARREF
-	if (ref($body) eq 'SCALAR') {
-		$output .= $$body;
-	} else {
-		$output .= $body;
-	}
+	my $output = $headers . $$bodyref;
 
 	# Send output to browser (unless we're in serious debug mode!)
 	unless ($ENV{CGI_APP_RETURN_ONLY}) {
@@ -202,12 +201,20 @@ sub cgiapp_prerun {
 }
 
 
+sub cgiapp_postrun {
+	my $self = shift;
+	my $bodyref = shift;
+
+	# Nothing to postrun, yet!
+}
+
+
 sub setup {
 	my $self = shift;
 
 	$self->start_mode('start');
 	$self->run_modes(
-		'start' => \&dump_html,
+		'start' => 'dump_html',
 	);
 }
 
@@ -229,8 +236,13 @@ sub dump {
 	my $self = shift;
 	my $output = '';
 
+	# Dump run-mode
+	my $current_runmode = $self->get_current_runmode();
+	$current_runmode = "" unless (defined($current_runmode));
+	$output .= "Current Run-mode: '$current_runmode'\n";
+
 	# Dump Params
-	$output .= "Query Parameters:\n";
+	$output .= "\nQuery Parameters:\n";
 	my @params = $self->query->param();
 	foreach my $p (sort(@params)) {
 		my @data = $self->query->param($p);
@@ -252,6 +264,10 @@ sub dump_html {
 	my $self = shift;
 	my $output = '';
 
+	# Dump run-mode
+	my $current_runmode = $self->get_current_runmode();
+	$output .= "<P>\nCurrent Run-mode: '<B>$current_runmode</B>'<BR>\n";
+
 	# Dump Params
 	$output .= "<P>\nQuery Parameters:<BR>\n<OL>\n";
 	my @params = $self->query->param();
@@ -305,14 +321,16 @@ sub header_type {
 	my $self = shift;
 	my ($header_type) = @_;
 
+        my @allowed_header_types = qw(header redirect none);
+
 	# First use?  Create new __HEADER_TYPE!
 	$self->{__HEADER_TYPE} = 'header' unless (exists($self->{__HEADER_TYPE}));
 
 	# If data is provided, set it!
 	if (defined($header_type)) {
 		$header_type = lc($header_type);
-		croak("Invalid header type '$header_type'.  Header type must be 'header' or 'redirect'")
-			unless(($header_type eq 'header') || ($header_type eq 'redirect'));
+		croak("Invalid header_type '$header_type'")
+			unless(grep { $_ eq $header_type } @allowed_header_types);
 		$self->{__HEADER_TYPE} = $header_type;
 	}
 
@@ -408,7 +426,7 @@ sub query {
 		my $new_query_obj;
 
 		# If data is provided, set it!  Otherwise, create a new one.
-		if (defined($query) && $query->isa('CGI')) {
+		if (defined($query)) {
 			$new_query_obj = $query;
 		} else {
 			$new_query_obj = $self->cgiapp_get_query();
@@ -532,11 +550,19 @@ sub _send_headers {
 	my $self = shift;
 	my $q = $self->query();
 
-	if ($self->header_type() =~ /redirect/i) {
+        my $header_type = $self->header_type();
+
+	if ($header_type eq 'redirect') {
 		return $q->redirect($self->header_props());
-	} else {
+	} elsif ($header_type eq 'header' ) {
 		return $q->header($self->header_props());
 	}
+
+        # croak() if we have an unknown header type
+        croak ("Invalid header_type '$header_type'") unless ($header_type eq "none");
+
+        # Do nothing if header type eq "none".
+        return "";
 }
 
 
@@ -838,12 +864,12 @@ TMPL_PATH - This optional parameter adds value to the load_tmpl()
 method (specified below).  This sets a path using HTML::Template's
 C<path> option when you call load_tmpl() to get your HTML::Template
 object.  This run-time parameter allows you to further encapsulate
-instantiating templates, providing potential for more reusability.
+instantiating templates, providing potential for more re-usability.
 
 PARAMS        - This parameter, if used, allows you to set a number 
 of custom parameters at run-time.  By passing in different 
 values in different instance scripts which use the same application 
-module you can achieve a higher level of reusability.  For instance, 
+module you can achieve a higher level of re-usability.  For instance, 
 imagine an application module, "Mailform.pm".  The application takes 
 the contents of a HTML form and emails it to a specified recipient.
 You could have multiple instance scripts throughout your site which 
@@ -931,7 +957,7 @@ method to store state information about the application to the server.
 =item cgiapp_init()
 
 If implemented, this method is called automatically right before the
-setup() method is called.  This method provides an optional initalization
+setup() method is called.  This method provides an optional initialization
 hook, which improves the object-oriented characteristics of 
 CGI::Application.  The cgiapp_init() method receives, as its parameters,
 all the arguments which were sent to the new() method.
@@ -1009,6 +1035,58 @@ run-mode of your application.  This can be done via the prerun_mode()
 method, which is discussed elsewhere in this POD.
 
 
+
+=item cgiapp_postrun()
+
+If implemented, this hook will be called after the run-mode method
+has returned its output, but before HTTP headers are generated.  This
+will give you an opportunity to modify the body and headers before they 
+are returned to the web browser.
+
+A typical use for this hook is pipelining the output of a CGI-Application 
+through a series of "filter" processors.  For example:
+
+  * You want to enclose the output of all your CGI-Applications in 
+    an HTML table in a larger page.
+
+  * Your run-modes return structured data (such as XML), which you
+    want to transform using a standard mechanism (such as XSLT).
+
+  * You want to post-process CGI-App output through another system,
+    such as HTML::Mason.
+
+  * You want to modify HTTP headers in a particular way across all
+    run-modes, based on particular criteria.
+
+The cgiapp_postrun() hook receives a reference to the output from
+your run-mode method, in addition to the CGI-App object.  A typical
+cgiapp_postrun() method might be implemented as follows:
+
+  sub cgiapp_postrun {
+    my $self = shift;
+    my $output_ref = shift;
+
+    # Enclose output HTML table
+    my $new_output = "<table border=1>";
+    $new_output .= "<tr><td> Hello, World! </td></tr>";
+    $new_output .= "<tr><td>". $$output_ref ."</td></tr>";
+    $new_output .= "</table>";
+
+    # Replace old output with new output
+    $output_ref = \$new_output;
+  }
+
+
+Obviously, with access to the CGI-App object you have full access to
+use all the methods normally available in a run-mode.  You could, for 
+example, use load_tmpl() to replace the static HTML in this example
+with HTML::Template.  You could change the HTTP headers (via 
+header_type() and header_props() methods) to set up a redirect.
+You could also use the objects properties to 
+apply changes only under certain circumstance, such as a in only 
+certain run-modes, and when a param() is a particular value.
+
+
 =item cgiapp_get_query()
 
 This method is called when CGI::Application retrieves the CGI query object.
@@ -1090,15 +1168,30 @@ Understanding this relationship is important if you wish to manipulate
 the HTTP header properly.
 
 
-=item header_type([<'header' || 'redirect'>])
+=item header_type([<'header' || 'redirect' || 'none'>])
 
     $webapp->header_type('redirect');
 
-The header_type() method expects to be passed either 'header' or 'redirect'.
+The header_type() method expects to be passed either 'header', 'redirect', or 'none'.
 This method specifies the type of HTTP headers which should be sent back to 
 the browser.  If not specified, defaults is 'header'.  See the 
 header section of L<CGI> for details.
 
+To perform a redirect using CGI::Application (and CGI.pm), you would
+do the following:
+
+  sub some_redirect_mode {
+    my $self = shift;
+    my $new_url = "http://site/path/doc.html";
+    $self->header_type('redirect');
+    $self->header_props(-url=>$new_url);
+    return "Redirecting to $new_url";
+  }
+
+If you wish to suppress HTTP headers entirely (as might be the case if 
+you're working in a slightly more exotic environment), you can set
+header_type() to "none".  This will completely hide headers.
+
 
 =item load_tmpl()
 
@@ -1113,7 +1206,7 @@ If tmpl_path() has been specified, load_tmpl() will set the
 HTML::Template C<path> option to the path provided.  This further
 assists in encapsulating template usage.
 
-The load_tmpl() method will pass any extra paramaters sent to it directly to 
+The load_tmpl() method will pass any extra parameters sent to it directly to 
 HTML::Template->new_file().  This will allow the HTML::Template object to be 
 further customized:
 
@@ -1123,7 +1216,7 @@ further customized:
     );
 
 If your application requires more specialized behavior than this, you are
-encoraged to override load_tmpl() by implementing your own load_tmpl() 
+encouraged to override load_tmpl() by implementing your own load_tmpl() 
 in your CGI::Application sub-class application module.
 
 
@@ -1196,7 +1289,7 @@ multiple boards, each with a different topic and set of administrators.
 The new() method provides a shortcut for specifying a number of run-time
 parameters at once.  Internally, CGI::Application calls the param() 
 method to set these properties.  The param() method is a powerful tool for 
-greatly increasing your application's reusability.
+greatly increasing your application's re-usability.
 
 
 =item query()
@@ -1310,10 +1403,10 @@ a run-mode with the reserved name "AUTOLOAD":
 	"AUTOLOAD" => \&catch_my_exception
   );
 
-Before CGI::Application calls croak() it will check for the existance 
+Before CGI::Application calls croak() it will check for the existence 
 of a run-mode called "AUTOLOAD".  If specified, this run-mode will in 
-involked just like a regular run-mode, with one exception:  It will 
-receive, as an argument, the name of the run-mode which involked it:
+invoked just like a regular run-mode, with one exception:  It will 
+receive, as an argument, the name of the run-mode which invoked it:
 
   sub catch_my_exception {
 	my $self = shift;
@@ -1374,7 +1467,7 @@ For example, consider:
 
 
 In this example, the web user will be forced into the "login" run-mode
-unless they have aleady logged in.  The prerun_mode() method permits
+unless they have already logged in.  The prerun_mode() method permits
 a scalar text string to be set which overrides whatever the run-mode
 would otherwise be.
 
@@ -1442,8 +1535,8 @@ for a company to develop your web site or individual developers to
 augment your team please contact us:
 
     The Erlbaum Group
-    250 East 31st Street, suite 6C
-    New York, NY 10016
+    1133 Broadway, suite 1321
+    New York, NY 10010
 
     Phone: 212-684-6161
     Fax: 212-684-6226
@@ -1469,6 +1562,7 @@ patches which have helped improve CGI::Application --
     Steve Comrie
     Darin McBride
     Eric Andreychek
+    Steve Hay
 
 
 Thanks also to all the members of the CGI-App mailing list!
